==> wmconfig-1.2.9.tar.bz2: wmconfig-1.2.9/gnome2.c <==
/*
 * Copyright (C) 2005-2006
 *	Tommy Scheunemann <net@arrishq.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>
#include <limits.h>
#include <strings.h>

#include "wmconfig.h"
#include "prototypes.h"
#include "package.h"

/* define default output dirs
 if the directory exists and there are
 contents, they will be replaced with
 Wmconfig entries
*/

/* Item Dir for Wmconfig entries */
#define GNOME_DIR ".local/share/applications"

/* Virtual Dir - needed for Gnome */
#define VFOLDER_DIR ".config/menus"

/* Configuration file for the panel */
#define INFO_FILE ".config/menus/wmconfig.menu"

extern const char *root_menu_name;

/* Fill the configuration file with contents the <INCLUDE> function is used to
 let the xml file read by the panel load the files
*/

static void fill_infofile(struct group *root, int level, const char *file, const char *itemdir)
{
 int i;
 int fd;
 FILE *f;
 struct item *item;
 if (root == (struct group *)NULL)
  return;

 /* Open the file and append the menu info at the end */

#ifdef BSD
 fd = open(file, O_APPEND | O_NOFOLLOW | O_RDWR, 0600);
#else
 fd = open(file, O_APPEND | O_RDWR, 0600);
#endif
 f = fdopen(fd, "w+");

 if (level>=0) {
  for (i=0; i<level; i++) {
   fprintf(f," ");
  }
  fprintf(f," <Menu>\n  <Name>%s</Name>\n", root->name);
 }
 item = root->items;
 while (item->type != 0) {
  if (item->type == ITEM_MENU) {
   fclose(f);
   fill_infofile(item->data, level+1, file, itemdir);
  } else if (item->type == ITEM_APP) {
   struct package *app;
   app = (struct package *)item->data;
   if (app != NULL) {
    for (i=0; i<level; i++) {
     fprintf(f," ");
    }

/* We don't write bash.desktop, we write Wmconfig_bash.desktop in case the user
 already have such an entry - easier for the difference what is written by
 Wmconfig.
 The string Wmconfig can be replaced giving the root_menu option
*/

    fprintf(f," <Include>\n");
    fprintf(f,"   <Filename>%s_%s.desktop</Filename>\n", single_string(root_menu_name), single_string(app->name));
    fprintf(f,"  </Include>\n");
   }
  }
  item++;
 }
 if (level>=0) {
  for (i=0; i<level; i++) {
   fprintf(f," ");
  }
  fprintf(f,"</Menu>\n");
 }
}

/* This creates the basic xml configuration file and fills it
with required contents - header */

static void create_infofile (const char *file, const char *itemdir, const char *writedir, struct group *root)
{
 int fd;
 FILE *f;

 /* Open and create the menu xml file */

#ifdef BSD
 fd = open(file, O_CREAT | O_EXCL | O_NOFOLLOW | O_RDWR, 0600);
#else
 fd = open(file, O_CREAT | O_EXCL | O_RDWR, 0600);
#endif
 f = fdopen(fd, "w+");
 if (f == (FILE *) NULL) {
  fprintf(stderr,"Could not create file %s\n", file);
  perror("Error code");
  return;
 }
 fprintf (f,"<?xml version=\"1.0\"?>\n");
 fprintf (f,"<Menu>\n");
 fprintf (f," <Name>Wmconfig</Name>\n");
 fprintf (f," <MergeFile type=\"parent\"/>");
 fclose (f);
 fill_infofile (root, 0, file, itemdir);
 fprintf (f,"</Menu>\n");
 fclose (f);
}

/* Create the .desktop files for proggies */

static void create_desktop_entry(const char *file, struct package *app)
{
    int fd;
    FILE *f;

    if (app == (struct package *)NULL) {
	return;
    }
    
    /* Remove the & at the end to prevent background starting */
    if (app->exec != NULL) {
	char *p;
	p = strrchr(app->exec, '&');
	if (p != NULL)
	    *p = '\0';
    }

    /*  At the very least we need a filename and command to execute:  */
    if (file == NULL || app->exec == NULL || app->restart)
	return;
	
#ifdef BSD
    fd = open(file, O_CREAT | O_EXCL | O_NOFOLLOW | O_RDWR, 0600);
#else
    fd = open(file, O_CREAT | O_EXCL | O_RDWR, 0600);
#endif
    f = fdopen(fd, "w+");
    if (f == (FILE *) NULL) {
	fprintf(stderr,"Could not create file %s\n", file);
	perror("Error code");
	return;
    }

    /*  Write the .desktop file:  */
    fprintf(f, "[Desktop Entry]\n");
    if (app->name != NULL)
	fprintf(f, "Name=%s\n", app->name);
    if (app->icon != NULL)
	fprintf(f, "Icon=%s\n", app->icon);
    if (app->description != NULL)
	fprintf(f, "Comment=%s\n", app->description);
    if (app->terminal != NULL)
	fprintf(f, "Terminal=true\n");
    fprintf(f, "Exec=%s\n", app->exec);
    fprintf(f, "Type=Application\n");
    
    /* Mimetype Support */
    if (app->mimetype != NULL)
     fprintf(f, "MimeType=%s;\n", app->mimetype);
    
    fclose(f);
}

/*  Like mkdir -p  */
static void make_directory_path(const char *path)
{
    char buf[PATH_MAX];
    int pi=0;
    int bi=0;
    while( path[pi] != '\0' ) {
        buf[bi] = path[pi];
        bi++ ; pi++ ;
        buf[bi] = '\0';
        if ( path[pi] == '\0' || path[pi] == '/' ) {
            mkdir(buf, 0700);
        }
    }
}

static void make_dir(struct group *root, const char *dir_name)
{
    struct item *item;
    struct stat st;
    char c_tmp[PATH_MAX];

    if (root == (struct group *)NULL)
	return;
						
    item = root->items;
    while (item->type != 0) {
	if (item->type == ITEM_APP) {
	    struct package *app;

	    app = (struct package *)item->data;
	    if (app->name && app->exec) {
		snprintf(c_tmp, PATH_MAX, "%s/%s_%s.desktop", single_string(dir_name), single_string(root_menu_name), single_string(app->name));
		
		/* Delete the file if it exists already */
		
		if (stat(c_tmp, &st) == 0) {
		    unlink(c_tmp);
		}
                create_desktop_entry(c_tmp, app);
	    }
	}
	item++;
    }
    /* second pass, recursive output... */
    item = root->items;

      while (item->type != 0) {
	if (item->type == ITEM_MENU) {
            struct group *tmp;

	    tmp = (struct group *)item->data;
	    snprintf(c_tmp, PATH_MAX, "%s/%s", dir_name, tmp->name);
	    make_dir(item->data, dir_name);
	}
	item++;
    }
}

/* Basic output function - launchers and the configuration file */

void output_gnome2(struct group *root)
{
 struct stat st;
 char root_dir[PATH_MAX];
 char config_file[PATH_MAX];
 char vfolder_dir[PATH_MAX];
 struct passwd *pw;
 setpwent();
 pw = getpwuid(getuid());
 if (pw == (struct passwd *)NULL) {
  fprintf(stderr, "Could not find out who you are (getpwnam failed)!\n");
  return;
 }
 endpwent();
 snprintf(vfolder_dir, PATH_MAX, "%s/%s", pw->pw_dir, VFOLDER_DIR);
 snprintf(config_file, PATH_MAX, "%s/%s", pw->pw_dir, INFO_FILE);
 snprintf(root_dir, PATH_MAX, "%s/%s", pw->pw_dir, GNOME_DIR);

/* Check if the directory for the desktop entries exist */ 

 if (stat(root_dir, &st) == 0) {
  if (!S_ISDIR(st.st_mode)) {
   fprintf(stderr, "Error: %s exists but it is not a directory\n", root_dir);
   return;
  }
 }
 
/* Check if the directory for the xml file exists */
 
 if (stat(vfolder_dir, &st) == 0) {
  if (!S_ISDIR(st.st_mode)) {
   fprintf(stderr, "Error: %s exists but it is not a directory\n", vfolder_dir);
   return;
  }
 }

/* Delete the xml file if it exists */

 if (stat(config_file, &st) == 0) {
  unlink(config_file);
 }

/* Create the required directories */

 make_directory_path(root_dir);
 make_directory_path(vfolder_dir);

/* Fill the menu directory */ 

 make_dir(root, root_dir);
 
/* Create xml configuration file for Gnome */

 create_infofile(config_file, root_dir, vfolder_dir, root);
}

==> wmconfig-1.3.1.tar.bz2: wmconfig-1.3.1/gnome2.c <==
/*
 * Copyright (C) 2005-2009
 *	Tommy Scheunemann <net@arrishq.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>
#include <limits.h>
#include <strings.h>

#include "wmconfig.h"
#include "prototypes.h"
#include "package.h"

/* define default output dirs
 if the directory exists and there are
 contents, they will be replaced with
 Wmconfig entries
*/

/* Item Dir for Wmconfig entries */
#define GNOME_DIR ".local/share/applications"

/* Virtual Dir - needed for Gnome */
#define VFOLDER_DIR ".config/desktop/menus/applications-merged"

/* Configuration file for the panel */
#define INFO_FILE ".config/desktop/menus/applications-merged/wmconfig.menu"

extern const char *root_menu_name;

/* Fill the configuration file with contents the <INCLUDE> function is used to
 let the xml file read by the panel load the files
*/

static void fill_infofile(struct group *root, int level, const char *file, const char *itemdir)
{
 int i;
 int fd;
 FILE *f;
 struct item *item;
 if (root == (struct group *)NULL)
  return;

 /* Open the file and append the menu info at the end */

#ifdef BSD
 fd = open(file, O_APPEND | O_NOFOLLOW | O_RDWR, 0600);
#else
 fd = open(file, O_APPEND | O_RDWR, 0600);
#endif
 f = fdopen(fd, "w+");

 if (level>=0) {
  for (i=0; i<level; i++) {
   fprintf(f," ");
  }
  fprintf(f," <Menu>\n  <Name>%s</Name>\n", root->name);
 }
 item = root->items;
 while (item->type != 0) {
  if (item->type == ITEM_MENU) {
   fclose(f);
   fill_infofile(item->data, level+1, file, itemdir);
  } else if (item->type == ITEM_APP) {
   struct package *app;
   app = (struct package *)item->data;
   if (app != NULL) {
    for (i=0; i<level; i++) {
     fprintf(f," ");
    }

/* We don't write bash.desktop, we write Wmconfig_bash.desktop in case the user
 already have such an entry - easier for the difference what is written by
 Wmconfig.
 The string Wmconfig can be replaced giving the root_menu option
*/

    fprintf(f," <Include>\n");
    fprintf(f,"   <Filename>%s_%s.desktop</Filename>\n", single_string(root_menu_name), single_string(app->name));
    fprintf(f,"  </Include>\n");
   }
  }
  item++;
 }
 if (level>=0) {
  for (i=0; i<level; i++) {
   fprintf(f," ");
  }
  fprintf(f,"</Menu>\n");
 }
}

/* This creates the basic xml configuration file and fills it
with required contents - header */

static void create_infofile (const char *file, const char *itemdir, const char *writedir, struct group *root)
{
 int fd;
 FILE *f;

 /* Open and create the menu xml file */

#ifdef BSD
 fd = open(file, O_CREAT | O_EXCL | O_NOFOLLOW | O_RDWR, 0600);
#else
 fd = open(file, O_CREAT | O_EXCL | O_RDWR, 0600);
#endif
 f = fdopen(fd, "w+");
 if (f == (FILE *) NULL) {
  fprintf(stderr,"Could not create file %s\n", file);
  perror("Error code");
  return;
 }
 fprintf (f,"<?xml version=\"1.0\"?>\n");
 fprintf (f,"<Menu>\n");
 fprintf (f," <Name>Wmconfig</Name>\n");
 fprintf (f," <MergeFile type=\"parent\"/>");
 fclose (f);
 fill_infofile (root, 0, file, itemdir);
 fprintf (f,"</Menu>\n");
 fclose (f);
}

/* Create the .desktop files for proggies */

static void create_desktop_entry(const char *file, struct package *app)
{
    int fd;
    FILE *f;

    if (app == (struct package *)NULL) {
	return;
    }
    
    /* Remove the & at the end to prevent background starting */
    if (app->exec != NULL) {
	char *p;
	p = strrchr(app->exec, '&');
	if (p != NULL)
	    *p = '\0';
    }

    /*  At the very least we need a filename and command to execute:  */
    if (file == NULL || app->exec == NULL || app->restart)
	return;
	
#ifdef BSD
    fd = open(file, O_CREAT | O_EXCL | O_NOFOLLOW | O_RDWR, 0600);
#else
    fd = open(file, O_CREAT | O_EXCL | O_RDWR, 0600);
#endif
    f = fdopen(fd, "w+");
    if (f == (FILE *) NULL) {
	fprintf(stderr,"Could not create file %s\n", file);
	perror("Error code");
	return;
    }

    /*  Write the .desktop file:  */
    fprintf(f, "[Desktop Entry]\n");
    if (app->name != NULL)
	fprintf(f, "Name=%s\n", app->name);
    if (app->icon != NULL)
	fprintf(f, "Icon=%s\n", app->icon);
    if (app->description != NULL)
	fprintf(f, "Comment=%s\n", app->description);
    if (app->terminal != NULL)
	fprintf(f, "Terminal=true\n");
    fprintf(f, "Exec=%s\n", app->exec);
    fprintf(f, "Type=Application\n");
    
    /* Mimetype Support */
    if (app->mimetype != NULL)
     fprintf(f, "MimeType=%s;\n", app->mimetype);
    
    fclose(f);
}

/*  Like mkdir -p  */
static void make_directory_path(const char *path)
{
    char buf[PATH_MAX];
    int pi=0;
    int bi=0;
    while( path[pi] != '\0' ) {
        buf[bi] = path[pi];
        bi++ ; pi++ ;
        buf[bi] = '\0';
        if ( path[pi] == '\0' || path[pi] == '/' ) {
            mkdir(buf, 0700);
        }
    }
}

static void make_dir(struct group *root, const char *dir_name)
{
    struct item *item;
    struct stat st;
    char c_tmp[PATH_MAX];

    if (root == (struct group *)NULL)
	return;
						
    item = root->items;
    while (item->type != 0) {
	if (item->type == ITEM_APP) {
	    struct package *app;

	    app = (struct package *)item->data;
	    if (app->name && app->exec) {
		snprintf(c_tmp, PATH_MAX, "%s/%s_%s.desktop", single_string(dir_name), single_string(root_menu_name), single_string(app->name));
		
		/* Delete the file if it exists already */
		
		if (stat(c_tmp, &st) == 0) {
		    unlink(c_tmp);
		}
                create_desktop_entry(c_tmp, app);
	    }
	}
	item++;
    }
    /* second pass, recursive output... */
    item = root->items;

      while (item->type != 0) {
	if (item->type == ITEM_MENU) {
            struct group *tmp;

	    tmp = (struct group *)item->data;
	    snprintf(c_tmp, PATH_MAX, "%s/%s", dir_name, tmp->name);
	    make_dir(item->data, dir_name);
	}
	item++;
    }
}

/* Basic output function - launchers and the configuration file */

void output_gnome2(struct group *root)
{
 struct stat st;
 char root_dir[PATH_MAX];
 char config_file[PATH_MAX];
 char vfolder_dir[PATH_MAX];
 struct passwd *pw;
 setpwent();
 pw = getpwuid(getuid());
 if (pw == (struct passwd *)NULL) {
  fprintf(stderr, "Could not find out who you are (getpwnam failed)!\n");
  return;
 }
 endpwent();
 snprintf(vfolder_dir, PATH_MAX, "%s/%s", pw->pw_dir, VFOLDER_DIR);
 snprintf(config_file, PATH_MAX, "%s/%s", pw->pw_dir, INFO_FILE);
 snprintf(root_dir, PATH_MAX, "%s/%s", pw->pw_dir, GNOME_DIR);

/* Check if the directory for the desktop entries exist */ 

 if (stat(root_dir, &st) == 0) {
  if (!S_ISDIR(st.st_mode)) {
   fprintf(stderr, "Error: %s exists but it is not a directory\n", root_dir);
   return;
  }
 }
 
/* Check if the directory for the xml file exists */
 
 if (stat(vfolder_dir, &st) == 0) {
  if (!S_ISDIR(st.st_mode)) {
   fprintf(stderr, "Error: %s exists but it is not a directory\n", vfolder_dir);
   return;
  }
 }

/* Delete the xml file if it exists */

 if (stat(config_file, &st) == 0) {
  unlink(config_file);
 }

/* Create the required directories */

 make_directory_path(root_dir);
 make_directory_path(vfolder_dir);

/* Fill the menu directory */ 

 make_dir(root, root_dir);
 
/* Create xml configuration file for Gnome */

 create_infofile(config_file, root_dir, vfolder_dir, root);
}
